@@ -1,7 +1,7 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE PatternGuards #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 
 module Network.Wai.Handler.Warp.HTTP2 (
@@ -32,7 +32,7 @@ import Network.Wai.Handler.Warp.Types
 ----------------------------------------------------------------
 
 http2 :: S.Settings -> InternalInfo -> Connection -> Transport -> Application -> SockAddr -> T.Handle -> ByteString -> IO ()
-http2 settings ii conn transport app origAddr th bs = do
+http2 settings ii conn transport app peersa th bs = do
     istatus <- newIORef False
     rawRecvN <- makeRecvN bs $ connRecv conn
     writeBuffer <- readIORef $ connWriteBuffer conn
@@ -52,10 +52,14 @@ http2 settings ii conn transport app origAddr th bs = do
           , confReadN             = recvN
           , confPositionReadMaker = pReadMaker ii
           , confTimeoutManager    = timeoutManager ii
+#if MIN_VERSION_http2(4,2,0)
+          , confMySockAddr        = connMySockAddr conn
+          , confPeerSockAddr      = peersa
+#endif
           }
     checkTLS
     setConnHTTP2 conn True
-    H2.run conf $ http2server settings ii transport origAddr app
+    H2.run H2.defaultServerConfig conf $ http2server settings ii transport peersa app
   where
     checkTLS = case transport of
         TCP -> return () -- direct
